/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * MessageExchangeSynchronizer.java - Initial Developper : ofabre
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.framework.listener;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import org.objectweb.petals.component.framework.PEtALSCFException;
import org.objectweb.petals.component.framework.util.Exchange;

/**
 * A message exchange synchronizer
 *
 * @author eBM WebSourcing
 * @since PCF 2.0
 *
 */
public class MessageExchangeSynchronizer {

    private Exchange exchange;

    private CountDownLatch latch;

    private boolean acknowledged = false;

    private boolean accepted = false;

    private boolean timeout = false;

    /**
     * Creates a new instance of {@link MessageExchangeSynchronizer}
     *
     */
    public MessageExchangeSynchronizer() {
        super();
        latch = new CountDownLatch(1);
    }

    /**
     *
     * @param timeout
     * @return
     * @throws InterruptedException
     * @throws PEtALSCFException
     */
    public Exchange accept(long timeout) throws InterruptedException,
            PEtALSCFException {
        if (isAcknowledged()) {
            throw new PEtALSCFException("You can't accept a response "
                    + "for an exchange that has been acknowledged !");
        }
        waitUntilTimeout(timeout);
        return exchange;
    }

    /**
     *
     * @param timeout
     * @throws InterruptedException
     */
    private void waitUntilTimeout(long timeout) throws InterruptedException {
        if (timeout != 0) {
            if (!latch.await(timeout, TimeUnit.MILLISECONDS)) {
                setTimeout(true);
            }
        } else {
            latch.await();
        }
    }

    public void setExchange(Exchange exchange) {
        this.exchange = exchange;
    }

    public void notifyExchange() {
        latch.countDown();
    }

    public void ackResponseProcessed() {
        this.acknowledged = true;
    }

    public Exchange getExchange() {
        return exchange;
    }

    public boolean isAcknowledged() {
        return acknowledged;
    }

    public boolean isAccepted() {
        return accepted;
    }

    public void setAccepted(boolean accepted) {
        this.accepted = accepted;
    }

    public boolean isTimeout() {
        return timeout;
    }

    public void setTimeout(boolean timeout) {
        this.timeout = timeout;
    }
}
